define(['underscore'], function(){
    /**
     * 支持命名空间的存储类
     * @class Storage
     * @requires underscore
     * @author shinvey
     * @version $Id
     *
     * 存储结构
     * key value
     * namespace ID1,ID2,ID3
     * namespace-ID1 {name:'', date:''}
     * namespace-ID2 {name:'', date:''}
     * namespace-ID3 {name:'', date:''}
     *
     * 构造函数
     * @param string namespace 可选，定义命名空间
     * @param string engine 可选，定义存储引擎local|session
     */
    function Storage(namespace, engine){
        if ( typeof _ == 'undefined' ) throw new Error('Storage depends on underscore');
        this.engine = engine || 'session';
        this.NS(namespace);
    };
    Storage.prototype = {
        //switch to new namespace
        NS: function(name){
            this.name = name || 'namespace';
            var store = this._storage().getItem(this.name);
            this.records = (store && store.split(",")) || [];
            return this;
        },
        /**
         * 返回存储对象
         * @access private
         * @returns void
         */
        _storage : function() {
            return this.engine == 'local' ? window.localStorage : window.sessionStorage;
        },
        /**
         * 更新或添加项
         * @param {string} key
         * @param {object} val
         */
        setItem : function(key, val){
            if ( !_.isUndefined(key) && !_.isUndefined(val) ) {
                this._storage().setItem(this.name+"-"+key, JSON.stringify(val));
                if ( !_.contains(this.records, String(key)) ) {
                    this.records.push(String(key));
                    this._save();
                }
            }
        },
        /**
         * 获取项
         * @param {string|integer} key 键名或键的索引
         * @return {object|null|string}
         */
        getItem : function(key){
            var result = this._storage().getItem(
                this.name +"-"+ (_.isNumber(key) ? this.key(key) : key)
            );
            if ( _.isEmpty(result) ) {
                return this.removeItem(key);
            }
            try {
                result = JSON.parse( result );
            } catch (ex) {
                if (ex instanceof SyntaxError){
                    return this.removeItem(key);//抛弃此数据
                }
                else
                    throw ex;
            }

            return result;
        },
        /**
         * 获取指定索引的键名
         * @param {int} idx
         * @returns {string}
         */
        key: function(idx){
            return this.records[idx];
        },
        /**
         * 移除项
         * @param {string|int} key
         */
        removeItem : function(key){
            key = _.isNumber(key) ? this.key(key) : key;
            this._storage().removeItem( this.name +"-"+ key );
            //去除primary key
            this.records = _.reject(this.records, function(record_id){return record_id == key;});
            this._save();
            return null;
        },
        /**
         * 读取所有项
         * @param {object|array} filter 过滤过则
         * @example getAll({offset: 0, limit: 10})
         * @return {object}
         */
        getAll : function(filter){
            var records = [],
                result = {},
                filter = filter || {};
            if ( typeof filter.offset != 'undefined' && filter.offset >= 0 ) {
                records = this.records.slice(filter.offset, filter.offset + filter.limit || 10);
            } else {
                records = this.records;
            }

            _.each(_.isArray(filter) ? filter : records, function(val, key){
                val && (result[val] = this.getItem(val));
            }, this);

            return result;
        },
        /**
         * 更新命名空间所属primary key列表
         * @access private
         * @return void
         */
        _save : function() {
            this._storage().setItem(this.name, this.records.join(","));
        },
        /**
         * 清理命名空间中的项列表数据
         * @return void
         */
        clear : function(){
            _(this.records).chain().map(function(key){ return this.removeItem(key); }, this);
            this._storage().removeItem(this.name);
        },
        total: function(){
            return this.records.length;
        }
    };
    return Storage;
});